@@ -35,6 +35,6 @@
 #define	DUID_UUID	4
 
 size_t duid_make(void *, const struct interface *, uint16_t);
-size_t duid_init(const struct interface *);
+size_t duid_init(struct dhcpcd_ctx *, const struct interface *);
 
 #endif
