@@ -110,7 +110,19 @@ int if_getssid_wext(const char *ifname, uint8_t *ssid);
 #define	SOL_NETLINK	270
 #endif
 
-/* Someone should fix kernel headers for clang alignment warnings . */
+/*
+ * Someone should fix kernel headers for clang alignment warnings.
+ * But this is unlikely.
+ * https://www.spinics.net/lists/netdev/msg646934.html
+ */
+
+#undef NLA_ALIGNTO
+#undef NLA_ALIGN
+#undef NLA_HDRLEN
+#define NLA_ALIGNTO		4U
+#define NLA_ALIGN(len)		(((len) + NLA_ALIGNTO - 1) & ~(NLA_ALIGNTO - 1))
+#define NLA_HDRLEN		((int) NLA_ALIGN(sizeof(struct nlattr)))
+
 #undef IFA_RTA
 #define IFA_RTA(r)  ((struct rtattr *)(void *)(((char *)(r)) \
 	+ NLMSG_ALIGN(sizeof(struct ifaddrmsg))))
@@ -130,6 +142,7 @@ int if_getssid_wext(const char *ifname, uint8_t *ssid);
 
 struct priv {
 	int route_fd;
+	int generic_fd;
 	uint32_t route_pid;
 };
 
@@ -156,7 +169,7 @@ static int if_addressexists(struct interface *, struct in_addr *);
 static const char *mproc =
 #if defined(__alpha__)
 	"system type"
-#elif defined(__arm__)
+#elif defined(__arm__) || defined(__aarch64__)
 	"Hardware"
 #elif defined(__avr32__)
 	"cpu family"
@@ -414,6 +427,12 @@ if_opensockets_os(struct dhcpcd_ctx *ctx)
 	if (getsockname(priv->route_fd, (struct sockaddr *)&snl, &len) == -1)
 		return -1;
 	priv->route_pid = snl.nl_pid;
+
+	memset(&snl, 0, sizeof(snl));
+	priv->generic_fd = if_linksocket(&snl, NETLINK_GENERIC, 0);
+	if (priv->generic_fd == -1)
+		return -1;
+
 	return 0;
 }
 
@@ -425,6 +444,7 @@ if_closesockets_os(struct dhcpcd_ctx *ctx)
 	if (ctx->priv != NULL) {
 		priv = (struct priv *)ctx->priv;
 		close(priv->route_fd);
+		close(priv->generic_fd);
 	}
 }
 
@@ -465,26 +485,27 @@ if_getnetlink(struct dhcpcd_ctx *ctx, struct iovec *iov, int fd, int flags,
 	};
 	ssize_t len;
 	struct nlmsghdr *nlm;
-	int r;
+	int r = 0;
 	unsigned int again;
+	bool terminated;
 
 recv_again:
-	if ((len = recvmsg(fd, &msg, flags)) == -1)
-		return -1;
-	if (len == 0)
-		return 0;
+	len = recvmsg(fd, &msg, flags);
+	if (len == -1 || len == 0)
+		return (int)len;
 
 	/* Check sender */
 	if (msg.msg_namelen != sizeof(nladdr)) {
 		errno = EINVAL;
 		return -1;
 	}
+
 	/* Ignore message if it is not from kernel */
 	if (nladdr.nl_pid != 0)
 		return 0;
 
-	r = 0;
 	again = 0;
+	terminated = false;
 	for (nlm = iov->iov_base;
 	     nlm && NLMSG_OK(nlm, (size_t)len);
 	     nlm = NLMSG_NEXT(nlm, len))
@@ -492,6 +513,7 @@ recv_again:
 		again = (nlm->nlmsg_flags & NLM_F_MULTI);
 		if (nlm->nlmsg_type == NLMSG_NOOP)
 			continue;
+
 		if (nlm->nlmsg_type == NLMSG_ERROR) {
 			struct nlmsgerr *err;
 
@@ -504,17 +526,25 @@ recv_again:
 				errno = -err->error;
 				return -1;
 			}
+			again = 0;
+			terminated = true;
 			break;
 		}
 		if (nlm->nlmsg_type == NLMSG_DONE) {
 			again = 0;
+			terminated = true;
 			break;
 		}
-		if (cb != NULL && (r = cb(ctx, cbarg, nlm)) != 0)
-			break;
+		if (cb == NULL)
+			continue;
+		if (nlm->nlmsg_seq != (uint32_t)ctx->seq && fd != ctx->link_fd)
+			logwarnx("%s: received sequence %u, expecting %d",
+			    __func__, nlm->nlmsg_seq, ctx->seq);
+		else
+			r = cb(ctx, cbarg, nlm);
 	}
 
-	if (r == 0 && again)
+	if ((again || !terminated) && (ctx != NULL && ctx->link_fd != fd))
 		goto recv_again;
 
 	return r;
@@ -982,33 +1012,41 @@ static int
 if_sendnetlink(struct dhcpcd_ctx *ctx, int protocol, struct nlmsghdr *hdr,
     int (*cb)(struct dhcpcd_ctx *, void *, struct nlmsghdr *), void *cbarg)
 {
-	int s, r;
+	int s;
 	struct sockaddr_nl snl = { .nl_family = AF_NETLINK };
 	struct iovec iov = { .iov_base = hdr, .iov_len = hdr->nlmsg_len };
 	struct msghdr msg = {
 	    .msg_name = &snl, .msg_namelen = sizeof(snl),
 	    .msg_iov = &iov, .msg_iovlen = 1
 	};
-	bool use_rfd;
-
-	use_rfd = (protocol == NETLINK_ROUTE && hdr->nlmsg_type != RTM_GETADDR);
+	struct priv *priv = (struct priv *)ctx->priv;
+	unsigned char buf[16 * 1024];
+	struct iovec riov = {
+		.iov_base = buf,
+		.iov_len = sizeof(buf),
+	};
 
 	/* Request a reply */
 	hdr->nlmsg_flags |= NLM_F_ACK;
 	hdr->nlmsg_seq = (uint32_t)++ctx->seq;
+	if ((unsigned int)ctx->seq > UINT32_MAX)
+		ctx->seq = 0;
 
 #ifdef PRIVSEP
 	if (ctx->options & DHCPCD_PRIVSEP && if_netlinkpriv(protocol, hdr))
 		return (int)ps_root_sendnetlink(ctx, protocol, &msg);
 #endif
 
-	if (use_rfd) {
-		struct priv *priv = (struct priv *)ctx->priv;
-
-		s = priv->route_fd;
-	} else {
-		if ((s = if_linksocket(&snl, protocol, 0)) == -1)
-			return -1;
+	switch (protocol) {
+	case NETLINK_ROUTE:
+		if (hdr->nlmsg_type != RTM_GETADDR) {
+			s = priv->route_fd;
+			break;
+		}
+		/* FALLTHROUGH */
+	case NETLINK_GENERIC:
+		s = priv->generic_fd;
+#if 0
 #ifdef NETLINK_GET_STRICT_CHK
 		if (hdr->nlmsg_type == RTM_GETADDR) {
 			int on = 1;
@@ -1018,22 +1056,17 @@ if_sendnetlink(struct dhcpcd_ctx *ctx, int protocol, struct nlmsghdr *hdr,
 				logerr("%s: NETLINK_GET_STRICT_CHK", __func__);
 		}
 #endif
+#endif
+		break;
+	default:
+		errno = EINVAL;
+		return -1;
 	}
 
-	if (sendmsg(s, &msg, 0) != -1) {
-		unsigned char buf[16 * 1024];
-		struct iovec riov = {
-			.iov_base = buf,
-			.iov_len = sizeof(buf),
-		};
+	if (sendmsg(s, &msg, 0) == -1)
+		return -1;
 
-		r = if_getnetlink(ctx, &riov, s, 0, cb, cbarg);
-	} else
-		r = -1;
-
-	if (!use_rfd)
-		close(s);
-	return r;
+	return if_getnetlink(ctx, &riov, s, 0, cb, cbarg);
 }
 
 #define NLMSG_TAIL(nmsg)						\
@@ -1598,9 +1631,11 @@ bpf_open(const struct interface *ifp,
 		struct sockaddr_ll sll;
 		struct sockaddr_storage ss;
 	} su = {
-		.sll.sll_family = PF_PACKET,
-		.sll.sll_protocol = htons(ETH_P_ALL),
-		.sll.sll_ifindex = (int)ifp->index,
+		.sll = {
+			.sll_family = PF_PACKET,
+			.sll_protocol = htons(ETH_P_ALL),
+			.sll_ifindex = (int)ifp->index,
+		}
 	};
 #ifdef PACKET_AUXDATA
 	int n;
@@ -1978,8 +2013,11 @@ if_setup_inet6(const struct interface *ifp)
 	int ra;
 	char path[256];
 
-	/* The kernel cannot make stable private addresses. */
-	if (if_disable_autolinklocal(ctx, ifp->index) == -1 && errno != ENODEV)
+	/* The kernel cannot make stable private addresses.
+	 * However, a lot of distros ship newer kernel headers than
+	 * the kernel itself so sweep that error under the table. */
+	if (if_disable_autolinklocal(ctx, ifp->index) == -1 &&
+	    errno != ENODEV && errno != ENOTSUP && errno != EINVAL)
 		logdebug("%s: if_disable_autolinklocal", ifp->name);
 
 	/*
