@@ -50,7 +50,9 @@
 #define UNUSED(a)		(void)(a)
 
 struct logctx {
+	char		 log_buf[BUFSIZ];
 	unsigned int	 log_opts;
+	FILE		*log_err;
 #ifndef SMALL
 	FILE		*log_file;
 #ifdef LOGERR_TAG
@@ -115,6 +117,7 @@ logprintdate(FILE *stream)
 __printflike(3, 0) static int
 vlogprintf_r(struct logctx *ctx, FILE *stream, const char *fmt, va_list args)
 {
+	FILE *err;
 	int len = 0, e;
 	va_list a;
 #ifndef SMALL
@@ -123,8 +126,9 @@ vlogprintf_r(struct logctx *ctx, FILE *stream, const char *fmt, va_list args)
 	bool log_tag;
 #endif
 
-	if ((stream == stderr && ctx->log_opts & LOGERR_ERR_DATE) ||
-	    (stream != stderr && ctx->log_opts & LOGERR_LOG_DATE))
+	err = ctx->log_err == NULL ? stderr : ctx->log_err;
+	if ((stream == err && ctx->log_opts & LOGERR_ERR_DATE) ||
+	    (stream != err && ctx->log_opts & LOGERR_LOG_DATE))
 	{
 		if ((e = logprintdate(stream)) == -1)
 			return -1;
@@ -132,8 +136,8 @@ vlogprintf_r(struct logctx *ctx, FILE *stream, const char *fmt, va_list args)
 	}
 
 #ifdef LOGERR_TAG
-	log_tag = ((stream == stderr && ctx->log_opts & LOGERR_ERR_TAG) ||
-	    (stream != stderr && ctx->log_opts & LOGERR_LOG_TAG));
+	log_tag = ((stream == err && ctx->log_opts & LOGERR_ERR_TAG) ||
+	    (stream != err && ctx->log_opts & LOGERR_LOG_TAG));
 	if (log_tag) {
 		if (ctx->log_tag == NULL)
 			ctx->log_tag = getprogname();
@@ -143,8 +147,8 @@ vlogprintf_r(struct logctx *ctx, FILE *stream, const char *fmt, va_list args)
 	}
 #endif
 
-	log_pid = ((stream == stderr && ctx->log_opts & LOGERR_ERR_PID) ||
-	    (stream != stderr && ctx->log_opts & LOGERR_LOG_PID));
+	log_pid = ((stream == err && ctx->log_opts & LOGERR_ERR_PID) ||
+	    (stream != err && ctx->log_opts & LOGERR_LOG_PID));
 	if (log_pid) {
 		if ((e = fprintf(stream, "[%d]", getpid())) == -1)
 			return -1;
@@ -201,7 +205,12 @@ vlogmessage(int pri, const char *fmt, va_list args)
 	    (pri <= LOG_ERR ||
 	    (!(ctx->log_opts & LOGERR_QUIET) && pri <= LOG_INFO) ||
 	    (ctx->log_opts & LOGERR_DEBUG && pri <= LOG_DEBUG)))
-		len = vlogprintf_r(ctx, stderr, fmt, args);
+	{
+		FILE *err;
+
+		err = ctx->log_err == NULL ? stderr : ctx->log_err;
+		len = vlogprintf_r(ctx, err, fmt, args);
+	}
 
 	if (!(ctx->log_opts & LOGERR_LOG))
 		return len;
@@ -361,6 +370,30 @@ logsettag(const char *tag)
 }
 #endif
 
+int
+loggeterrfd(void)
+{
+	struct logctx *ctx = &_logctx;
+	FILE *err = ctx->log_err == NULL ? stderr : ctx->log_err;
+
+	return fileno(err);
+}
+
+int
+logseterrfd(int fd)
+{
+	struct logctx *ctx = &_logctx;
+
+	if (ctx->log_err != NULL)
+		fclose(ctx->log_err);
+	if (fd == -1) {
+		ctx->log_err = NULL;
+		return 0;
+	}
+	ctx->log_err = fdopen(fd, "a");
+	return ctx->log_err == NULL ? -1 : 0;
+}
+
 int
 logopen(const char *path)
 {
@@ -369,6 +402,8 @@ logopen(const char *path)
 	/* Cache timezone */
 	tzset();
 
+	(void)setvbuf(stderr, ctx->log_buf, _IOLBF, sizeof(ctx->log_buf));
+
 	if (path == NULL) {
 		int opts = 0;
 
