@@ -296,12 +296,11 @@ ps_inet_signalcb(int sig, void *arg)
 {
 	struct dhcpcd_ctx *ctx = arg;
 
-	/* Ignore SIGINT, respect PS_STOP command or SIGTERM. */
-	if (sig == SIGINT)
+	if (sig != SIGTERM)
 		return;
 
 	shutdown(ctx->ps_inet_fd, SHUT_RDWR);
-	eloop_exit(ctx->eloop, sig == SIGTERM ? EXIT_SUCCESS : EXIT_FAILURE);
+	eloop_exit(ctx->eloop, EXIT_SUCCESS);
 }
 
 ssize_t
